import { useState } from 'react';
import AIWidgetIcon from '../../../svgs/Icons/AIWidgets/AIWidget';
import { IChatMessage } from '../../../../../Views/Base/BaseViewModel';
import { stringifyError } from '../../../../../../utils/parse';
import ErrorMessageIcon from './ErrorMessageIcon';
import { AIMessageWidget } from './AIMessageWidget';

interface MessageWidgetProps {
  // https://api.openai.com/v1/chat/completions'
  // api: 'v1/chat/completions',
  // servername: 'https://myproxy.com',
  // https://func270519800.azurewebsites.net/api/TranslateTextToEng
  servername: string;
  api: string;
  port: string;
  sessionToken: string;
}
export default function UseDefaultAITranslateWidget({
  servername,
  api,
  port,
  sessionToken,
}: MessageWidgetProps): AIMessageWidget {
  const [errorMessage, setErrorMessage] = useState<string>('');

  // eslint-disable-next-line @typescript-eslint/no-unused-vars,@typescript-eslint/no-empty-function
  const fileToWidget = (file: File, context: IChatMessage[]): void => {};

  const renderWidget = (): JSX.Element => {
    if (errorMessage && errorMessage.length > 0) {
      const errorsDescriptions:
        | { title: string; action: () => void }[]
        | undefined = [];

      return (
        <ErrorMessageIcon
          errorMessageText={errorMessage}
          errorsDescriptions={errorsDescriptions}
        />
      );
    }

    return <AIWidgetIcon applyZoom color="green" />;
  };

  // async function getData(
  //   textToSend: string,
  //   dialogMessages: ChatCompletionRequestMessage[],
  // ): Promise<string> {
  //   //
  //   const apiEndpoint = 'https://api.openai.com/v1/chat/completions';
  //   const { apiKey } = QBConfig.configAIApi.AIAnswerAssistWidgetConfig; // Замените на ваш реальный ключ API
  //   const model = 'gpt-3.5-turbo';
  //   const requestOptions = {
  //     method: 'POST',
  //     headers: {
  //       'Content-Type': 'application/json',
  //       Authorization: `Bearer ${apiKey}`,
  //     },
  //     body: JSON.stringify({
  //       messages: [...dialogMessages, { role: 'user', content: textToSend }],
  //       model,
  //       temperature: 0.5,
  //     }),
  //   };
  //
  async function getData(
    prompt: string,
    dialogMessages: IChatMessage[],
  ): Promise<string> {
    let outputMessage = '';
    const apiEndpoint = `${servername}${port}${api}`;
    const apiKey = sessionToken; // Замените на ваш реальный ключ API
    const model = 'gpt-3.5-turbo';

    const requestOptions = {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${apiKey}`,
      },
      body: JSON.stringify({
        messages: [...dialogMessages, { role: 'user', content: prompt }],
        model,
        temperature: 0.5,
      }),
    };

    //
    try {
      const response = await fetch(apiEndpoint, requestOptions);
      const data = await response.json();

      outputMessage = data.choices[0].message?.content || '';
    } catch (err) {
      outputMessage = stringifyError(err);
      setErrorMessage(outputMessage);
    }

    return outputMessage;
  }

  const [textFromWidgetToContent, setTextFromWidgetToContent] = useState('');
  // const textToWidget = (value: string, context: IChatMessage[]): void => {
  //   if (value && value.length > 0) {
  //     // eslint-disable-next-line promise/catch-or-return
  //     getOpenAIApiData(value, context as ChatCompletionRequestMessage[]).then(
  //       // eslint-disable-next-line promise/always-return
  //       (data) => {
  //         setTextFromWidgetToContent(data);
  //       },
  //     );
  //   }
  // };
  const textToWidget = async (
    textToSend: string,
    context: IChatMessage[],
    additionalSettings?: { [key: string]: any },
  ): Promise<string> => {
    if (textToSend && textToSend.length > 0) {
      // eslint-disable-next-line no-return-await
      let prompt = `Please, translate the next text in english and give me just only translated text. Text to translate is: "${textToSend}"`;
      const { language } = additionalSettings || {};

      if (language) {
        prompt = `Please, translate the next text in ${
          language as string
        } and give me just only translated text. Text to translate is: "${textToSend}"`;
      }

      // eslint-disable-next-line no-return-await
      return await getData(prompt, context).then((data) => {
        setTextFromWidgetToContent(data);

        return data;
      });
    }

    return '';
  };

  return {
    textToContent: textFromWidgetToContent,
    renderWidget,
    textToWidget,
  };
}
